"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.injectScriptEventRelayScript = exports.replaceUrlsInStack = exports.attachBrowserScriptRelay = void 0;
const url_1 = require("url");
const ws_1 = __importDefault(require("ws"));
const utils_1 = require("../utils");
const cyclicJson_1 = require("./cyclicJson");
function attachBrowserScriptRelay(server, relay) {
    const routes = new Map();
    const wsServer = new ws_1.default.Server({ noServer: true });
    wsServer.on('connection', socket => {
        socket.on('message', message => {
            const [route, data] = (0, cyclicJson_1.parseCyclicJson)(message.toString());
            const handler = routes.get(route);
            if (handler) {
                handler({
                    ...data,
                    file: urlToFilePath(data.url),
                    stack: replaceUrlsInStack(relay, data.stack),
                    timestamp: new Date(data.timestamp),
                });
            }
        });
    });
    server.on('upgrade', (request, socket, head) => {
        wsServer.handleUpgrade(request, socket, head, socket => {
            wsServer.emit('connection', socket, request);
        });
    });
    function defineHandler(route, handler) {
        routes.set(route, handler);
    }
    defineHandler('connection', (event) => {
        relay.emitScriptEvent('connection', event);
    });
    defineHandler('console', (event) => {
        const args = event.args.map(decodeUnserializableValue);
        const data = { ...event, type: 'console', args };
        relay.emitScriptEvent('console', data);
    });
    defineHandler('document', (event) => {
        relay.emitScriptEvent('document', event);
    });
    defineHandler('error', (event) => {
        relay.emitScriptEvent('error', event);
    });
    defineHandler('window', (event) => {
        relay.emitScriptEvent('window', event);
    });
    function urlToFilePath(url) {
        if (!url)
            return url;
        try {
            new url_1.URL(url);
        }
        catch (e) {
            if (e.name !== 'TypeError')
                throw e;
            return null;
        }
        return relay.urlPathToFilePath(new url_1.URL(url).pathname);
    }
}
exports.attachBrowserScriptRelay = attachBrowserScriptRelay;
function replaceUrlsInStack(relay, stack) {
    if (!stack)
        return stack;
    return (stack
        .replace(/(?:^|\b)\S*@http:\/\/[^/]+\/__p5_server_static\/console-relay(?:\.min)\.js:\d+:\d+\n/g, '')
        .replace(/(?:^|\b) *at[^\n]+?http:\/\/[^/]+\/__p5_server_static\/console-relay(?:\.min)\.js:\d+:\d+\n/g, '')
        .replace(/ +at[^\n]+?\(http:\/\/[^/]+\/__p5_server_static\/console-relay(?:\.min)\.js:\d+:\d+\)\n/gm, '')
        .replace(/\bhttps?:\/\/(?:localhost|127\.0\.0\.1)(?::\d+)?\/[^\s:]+/g, url => relay.serverUrlToFileUrl(url) || url));
}
exports.replaceUrlsInStack = replaceUrlsInStack;
function injectScriptEventRelayScript(html) {
    return (0, utils_1.addScriptToHtmlHead)(html, '/__p5_server_static/console-relay.min.js');
}
exports.injectScriptEventRelayScript = injectScriptEventRelayScript;
const serializationPrefix = '__p5_server_serialization_:';
function decodeUnserializableValue(value) {
    return typeof value === 'string' && value.startsWith(serializationPrefix)
        ? eval(value.slice(serializationPrefix.length))
        : value;
}
